/* 
 * File:   my_defines.h
 * Author: arne
 *
 * Created on 15 de Outubro de 2022, 12:54
 */

#ifndef MY_DEFINES_H
#define	MY_DEFINES_H

//******************************************************************************
// Se nenhuma frequência foi definida, assume 4MHz
// Esta definiçõeo é exigida para calibrar as funções __delay_us() e __delay_ms()
//******************************************************************************
#define _XTAL_FREQ 4000000UL

//******************************************************************************
// Arquivos incluídos no módulo
//******************************************************************************
#include <xc.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

//******************************************************************************
// Definições importantes do módulo
//******************************************************************************

//*****************************************************************************
// Tipos definidos pelo usuário
//*****************************************************************************
#ifndef _MYDEFS_
#define _MYDEFS_
typedef unsigned char byte;
typedef unsigned char Uint8;
typedef unsigned short Uint16;
#endif

#define TRUE 			1
#define FALSE 			0
#define CR				0x0D
#define LF				0x0A
#define ACK				0x06

//#define __DEBUG__
//#define __INC_CRC__       //inclui módulo de calculo para CRC CCITT

//*******************************************************************************
// Definições para uso dos pinos de I/O
//*******************************************************************************
#define TECLAS          PORTA
#define LEDS            PORTB

#define LED1            RB6
#define LED2            RB5
#define LED3            RB4

//*******************************************************************************
// Definições para uso do protocolo
//*******************************************************************************
#define HEADER			'@'				// header do protocolo
#define NR_DISP			0x01			// nr do dispositivo na rede!

#define POSHEADER		0x00			// posições e tamanho final do buffer
#define POSDISPS		0x01
#define POSQTBYTES		0x02
#define POSDATA			0x03

#define TAMDATA			0x0E			// apenas 14 bytes - LCD
#define TAMBUF			(0x03 + TAMDATA + 0x01)

//*******************************************************************************
// Definições para uso do MAX7219
//*******************************************************************************
#define TAM_BUFFER 72

//******************************************************************************
// Variáveis globais a todos os módulos
//******************************************************************************
uint8_t buffer[TAM_BUFFER];     //buffer para bytes do display!
char TxRxbuf[TAMBUF];           //buffer de envio/recepção
char p[15];

volatile uint8_t autoriza;
volatile uint8_t counter;
volatile uint8_t prima;
volatile uint8_t envio;
volatile uint8_t mytimeout;
volatile uint8_t timeout_aut;
volatile uint8_t toTerm;

#ifdef __DEBUG__
volatile uint8_t debug;
#endif

#endif	/* MY_DEFINES_H */

