//******************************************************************************
// Arquivo pic_comm.c - tratamento da USART do PIC by Arne
// Desenvolvido por Eng. Márcio José Soares
//******************************************************************************

//******************************************************************************
// O Copyright deste programa está reservado para Márcio José Soares e seu
// conteúdo está protegido pela Lei de Direitos Autorais LEI N° 9.610, de
// 19 de Fevereiro de 1998. É estritamente proibida a reprodução total ou
// parcial dos conteúdos aqui apresentados sem prévia autorização, por escrito,
// do autor.
//******************************************************************************

//******************************************************************************
// Arquivos incluídos no módulo
//******************************************************************************
#include <xc.h>

//******************************************************************************
// Definições importantes do módulo
//******************************************************************************

#ifndef byte
#define byte uint8_t
#endif

#ifndef LEDS
#define LEDS    PORTB
#endif

//******************************************************************************
// Define serial em uso
//******************************************************************************
#ifndef _SERIAL_H_
#define _SERIAL_H_
#endif

//******************************************************************************
// Define baud rate, frequencia do oscilador e número de bits
//******************************************************************************
#define BAUD 9600
#define FOSC 4000000L
#define NINE 0          //1 se usar 9 bits, 0 se usar 8 - bit TX9 em TXSTA

//******************************************************************************
// Prepara divisor para calculo do valor no registrador SPBRG
//******************************************************************************
#define DIVIDER ((int)((FOSC/(16UL * BAUD)) -1))

//******************************************************************************
// Configura High speed - bit BRGH em TXSTA
//******************************************************************************
#define HIGH_SPEED 1

//******************************************************************************
// Prepara nr de bits - influencia em TXSTA
//******************************************************************************
#if NINE == 1
#define NINE_BITS 0x40
#else
#define NINE_BITS 0x00
#endif

//******************************************************************************
// Prepara high speed - incluencia em TXSTA
//******************************************************************************
#if HIGH_SPEED == 1
#define SPEED 0x04
#else
#define SPEED 0x00
#endif

//******************************************************************************
// Variáveis externas do módulo
//******************************************************************************
volatile uint8_t autoriza;
extern volatile uint8_t envio;

//******************************************************************************
// Funções externas do módulo
//******************************************************************************
extern void trata_rx(char c);

//******************************************************************************
// Funções do módulo
//******************************************************************************

//******************************************************************************
// Função - init_UART - Inicia a usart do PIC, conforme definições em pic_comm.h
//
// Entradas - nenhuma
// Saídas   - nenhuma
//******************************************************************************
void init_usart(void){

    TRISB1 = 1;                             //RX 16F6X8
    TRISB2 = 0;                             //TX 16F6X8
    
    SPBRG = DIVIDER;                        //seta velocidade
    TXSTA = (SPEED|NINE_BITS|0x20);         //numero de bits
    
    RCSTA = (0x80|0x10);                    //habilita recepção
    PIE1 |= 0x20;                           //habilita int recep.
    PIR1 &= ~0x20;                          //limpa flag de recepção
}

//******************************************************************************
// Função - putchar - saída de apenas um único byte
//
// Entradas - nenhuma
// Saídas   - nenhuma
//******************************************************************************
void usart_putch(unsigned char mbyte){

    while(!TXIF);               //aguarda registrador estar vazio
    TXREG = mbyte;              //envia byte

}

//******************************************************************************
// Função - putchar str - envia uma string
//
// Entradas - nenhuma
// Saídas   - nenhuma
//******************************************************************************
void usart_putchSTR(char *mystr){
    
    while(*mystr){
        usart_putch((unsigned char)*mystr);
        mystr++;
    }
}

//******************************************************************************
// Função - getchar - retorna um único byte
//
// Entradas - nenhuma
// Saídas   - nenhuma
//******************************************************************************
unsigned char usart_getch(void){

    while(!RCIF);               //espera até que registrador tenha um byte
    return RCREG;               //retorna byte recebido
}

//******************************************************************************
// Função - getchar - retorna um único caracter
//
// Entradas - nenhuma
// Saídas   - nenhuma
//******************************************************************************
unsigned char usart_getche(void){

    unsigned char c;

    usart_putch(c = usart_getch());       //pega caracter
    return c;                   //retorna caracter recebido
}

//*****************************************************************************
// Função trata int comm - testa e responde interrup��o de recep��o da USART
//
// Entradas - nenhuma
// Saídas   - caracter recebido
//*****************************************************************************
void trata_int_comm(void){

    byte c;
    
    while(!RCIF);           //aguarda receber
    c = RCREG;              //pega dado
    
    if(!autoriza && !envio)
        trata_rx(c);        //trata dado recebido
    
    while(!TXIF);           //aguarda registrador estar vazio
    TXREG = c;              //eco
    
    while(!TXIF);           //aguarda registrador estar vazio
    TXREG = 0x08;           //apaga �ltimo teclado
}

