
//******************************************************************************
// Arquivos incluídos no módulo
//******************************************************************************
#include "ring_comm.h"
#include "my_defines.h"

//*******************************************************************************
// Variáveis externas do módulo
//*******************************************************************************
extern char TxRxbuf[TAMBUF];
extern volatile uint8_t autoriza;
extern volatile uint8_t counter;
extern volatile uint8_t prima;
extern volatile uint8_t mytimeout;
extern volatile uint8_t timeout_aut;

//*******************************************************************************
// Funções externas do módulo
//*******************************************************************************
extern void usart_putch(unsigned char mbyte);

//*******************************************************************************
// Funções do módulo
//*******************************************************************************

//*******************************************************************************
// Função trata_rx - trata caracter recebido inserindo o mesmo no buffer
//
// Entradas - caracter a ser tratado
// Saídas   - nenhuma (altera var global avisando que tem dados no buffer)
//*******************************************************************************
void trata_rx(char c){

	if(!prima){									// é o primeiro caracter?
		if(c == HEADER){						// é header?
			prima = 1;
			TxRxbuf[counter++] = c;				// guarda
            timeout_aut = 0;
            mytimeout = 0;
            timeout_aut = 1;      
		}
		else{
			prima = 0;
			counter = 0;
			autoriza = 0;
            timeout_aut = 0;
            mytimeout = 0;
		}
	}
	else{										// já recebeu header
		if(counter == 1){						// agora recebe nr do dispositivos
			TxRxbuf[counter++] = c;				// apenas guarda
            timeout_aut = 0;
            mytimeout = 0;
            timeout_aut = 1; 
		}
		else if(counter == 2){					// agora recebe quantidade de caracteres de dados
			if(c >= 1 && c<=TAMDATA){			// quantidade dentro da faixa?
				TxRxbuf[counter++] = c;			// guarda
                timeout_aut = 0;
                mytimeout = 0;
                timeout_aut = 1; 
			}
			else{								// fora da faixa!
				prima = 0;
				autoriza = 0;
				counter = 0;
                timeout_aut = 0;
                mytimeout = 0;
			}
		}
		else{											// preparado para receber dados
			TxRxbuf[counter++] = c;						// recebe os dados
            timeout_aut = 0;
            mytimeout = 0;
            timeout_aut = 1; 
			if((counter - 3) >= TxRxbuf[POSQTBYTES]){	// se passou
				autoriza = 1;
				prima = 0;
				counter = 0;
                timeout_aut = 0;
                mytimeout = 0;
			}
		}
	}
}

//*******************************************************************************
// Função valida - testa buffer recebido e toma ação
//                 se dados são para este terminal, executa!
//                 do contrário, passa a frente!
//
// Entradas - buffer para inserir resposta (passagem por referência)
// Saídas   - 1 se dados são para este terminal; 0 do contrário
//*******************************************************************************
byte valida(char dadosTerm[], char data[]){

	uint8_t ret = 0;
	uint8_t nrdisp = (TxRxbuf[POSDISPS] & 0x0F);

	if(nrdisp == NR_DISP){									 		// se dado é para este dispositivo
		if(TxRxbuf[POSQTBYTES] != 0x01 && TxRxbuf[POSDATA] != ACK){	// se não é retorno de ACK
			dadosTerm[0] = ((TxRxbuf[POSDISPS] & 0xF0) >> 0x04); 	// pega sender
			dadosTerm[1] = TxRxbuf[POSQTBYTES];						// pega quantidade de bytes enviados
			for(uint8_t i=0;i<TxRxbuf[POSQTBYTES];i++){				// pega dados enviados
				data[i] = TxRxbuf[POSDATA+i];						// transfere
			}
            uint8_t aux = TxRxbuf[POSQTBYTES];
            data[aux] = '\0';                                       // fecha string
			ret = 0x01;												// retorna válido
		}
	}
	else{                                                           // dado não é desse dispositivo
		if((((TxRxbuf[POSDISPS] & 0xF0) >> 0x04) != NR_DISP) &&
               (((TxRxbuf[POSDISPS] & 0xF0) >> 0x04) != 0)){        // se nr do sender não é o mesmo, do contrário morre aqui
			for(uint8_t i=0;i<(3+TxRxbuf[POSQTBYTES]);i++)			// envia dado para próximo dispositivo
				usart_putch(TxRxbuf[i]);                            // dispara envio
		}
	}

	return(ret);
}